/*!\file ExponentialVariogram.c
 * \brief: implementation of the ExponentialVariogram object
 */

#ifdef HAVE_CONFIG_H
	#include <config.h>
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include <stdio.h>
#include <string.h>
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../include/include.h"

/*ExponentialVariogram constructors and destructor*/
/*FUNCTION ExponentialVariogram::ExponentialVariogram(){{{1*/
ExponentialVariogram::ExponentialVariogram(){
	this->nugget = 0.2;
	this->sill   = 1;
	this->range  = SQRT3;
	return;
}
/*}}}*/
/*FUNCTION ExponentialVariogram::ExponentialVariogram(Options* options){{{1*/
ExponentialVariogram::ExponentialVariogram(Options* options){

	/*Defaults*/
	this->nugget = 0.2;
	this->sill   = 1;
	this->range  = SQRT3;

	/*Overwrite from options*/
	if(options->GetOption("nugget")) options->Get(&this->nugget,"nugget");
	if(options->GetOption("sill"))   options->Get(&this->sill,"sill");
	if(options->GetOption("range"))  options->Get(&this->range,"range");

	/*Checks*/
	if(nugget==sill) _error_("nugget and sill cannot be equal (constant semivariogram not allowed)");
}
/*}}}*/
/*FUNCTION ExponentialVariogram::~ExponentialVariogram(){{{1*/
ExponentialVariogram::~ExponentialVariogram(){
	return;
}
/*}}}*/

/*Object virtual functions definitions:*/
/*FUNCTION ExponentialVariogram::Echo {{{1*/
void ExponentialVariogram::Echo(void){
	printf("ExponentialVariogram\n");
	printf("   nugget: %g\n",this->nugget);
	printf("   sill  : %g\n",this->sill);
	printf("   range : %g\n",this->range);
}
/*}}}*/

/*Variogram function*/
/*FUNCTION ExponentialVariogram::SemiVariogram{{{1*/
double ExponentialVariogram::SemiVariogram(double deltax,double deltay){
	/*http://en.wikipedia.org/wiki/Variogram*/
	double h,a,gamma;

	/*Calculate length*/
	h=sqrt(pow(deltax,2.)+pow(deltay,2.));

	/*return semi-variogram*/
	a     = 1./3.;
	gamma = (sill-nugget)*(1-exp(-h/(a*range))) + nugget;
	return gamma;
}
/*}}}*/
