/*!\file SphericalVariogram.c
 * \brief: implementation of the SphericalVariogram object
 */

#ifdef HAVE_CONFIG_H
	#include <config.h>
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include <stdio.h>
#include <string.h>
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../include/include.h"

/*SphericalVariogram constructors and destructor*/
/*FUNCTION SphericalVariogram::SphericalVariogram(){{{1*/
SphericalVariogram::SphericalVariogram(){
	this->nugget = 0.2;
	this->sill   = 1;
	this->range  = SQRT3;
	return;
}
/*}}}*/
/*FUNCTION SphericalVariogram::SphericalVariogram(Options* options){{{1*/
SphericalVariogram::SphericalVariogram(Options* options){

	/*Defaults*/
	this->nugget = 0.2;
	this->sill   = 1;
	this->range  = SQRT3;

	/*Overwrite from options*/
	if(options->GetOption("nugget")) options->Get(&this->nugget,"nugget");
	if(options->GetOption("sill"))   options->Get(&this->sill,"sill");
	if(options->GetOption("range"))  options->Get(&this->range,"range");

	/*Checks*/
	if(nugget==sill) _error_("nugget and sill cannot be equal (constant semivariogram not allowed)");
}
/*}}}*/
/*FUNCTION SphericalVariogram::~SphericalVariogram(){{{1*/
SphericalVariogram::~SphericalVariogram(){
	return;
}
/*}}}*/

/*Object virtual functions definitions:*/
/*FUNCTION SphericalVariogram::Echo {{{1*/
void SphericalVariogram::Echo(void){
	printf("SphericalVariogram\n");
	printf("   nugget: %g\n",this->nugget);
	printf("   sill  : %g\n",this->sill);
	printf("   range : %g\n",this->range);
}
/*}}}*/

/*Variogram function*/
/*FUNCTION SphericalVariogram::SemiVariogram{{{1*/
double SphericalVariogram::SemiVariogram(double deltax,double deltay){
	/*http://en.wikipedia.org/wiki/Variogram*/
	double h,gamma;

	/*Calculate length square*/
	h=sqrt(pow(deltax,2.)+pow(deltay,2.));

	/*return semi-variogram*/
	if(h<=range)
	 gamma = (sill-nugget)*( (3*h)/(2*range) - pow(h,3.)/(2*pow(range,3.)) ) + nugget;
	else
	 gamma = sill;

	return gamma;
}
/*}}}*/
